# -*- coding: iso-8859-15 -*-
#    Computation of total rewards depending on parameter (a) of
#    the best-fitting Zipf's law for the distribution of rewards,
#    number of people (N) rewarded above a significant level (S)
#    and minimal reward (M).
#    This program computes the cost for rewarding exactly N people
#    above S. In Sharing, we impose an extra condition of the cost being
#    constant when a varies, which leads to rewarding more or less than
#    N people at level S when a varies.
#    Note that parameter (a) can result of the directly observed usage or
#    of a non-proportional power law function being applied to observed usage.
#
#    Copyright (C) 2010-2011 - Philippe Aigrain
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from pylab import *

def harmonic(s,n):
  k=0.0;
  for i in arange (1.0,n+1.0,1.0):
    k += pow(i,-s);
  return k;  

def main(argv):
  if len(argv) < 5:
    print ("Usage: %s <number N of people above S><S reward><M reward><Zipf's law parameter>") % argv[0]
    print "Total reward using model and formula in Sharing and the Creative Economy"
    sys.exit(0)
  N = float(argv[1])
  S = float(argv[2])
  M = float(argv[3])
  a = float(argv[4])

# Computation of the total number N2 of people rewarded
# using formula N2 = exp[log(N)+((1/a)log(S/M))] (p. 195 of book)
  N2 = int(exp(log(N)+((1.0/a)*log(S/M))))

# Normalization of cost to the level corresponding to a = 1.0
# Direct result for the total cost
  print ("Total reward: %d") % (M * harmonic(a,N2) * pow(N2,a))

if __name__ == '__main__':
  main(sys.argv)
